#include <iostream>
#include <cstring>
#include "videoFormats.h"

#include <theora/codec.h>


#define SCALEBITS 8
#define ONE_HALF  (1 << (SCALEBITS - 1))
#define FIX(x)    ((int) ((x) * (1L<<SCALEBITS) + 0.5))

RGBPlane::RGBPlane(uint32 _width, uint32 _height)
{
  width = _width;
  height = _height;
  stride = _width;

  format = rgba;

  plane = new uint8[width*height*4];

}

bool RGBPlane::load(const std::string& filename, uint32 _width, uint32 _height, bool useBiggest)
{

  std::string::size_type suffixStart(filename.find_last_of('.'));

  if (suffixStart == std::string::npos) {
    std::cerr << "RGBPlane::load: Can not identify suffix of <"<<filename<<">\n";
    return(false);
  }

  std::string suffix(filename.substr(suffixStart+1));

  gdImagePtr im(0);

  FILE* in(0);
  in = fopen(filename.c_str(), "rb");

  if (in == 0) {
    std::cerr << "RGBPlane::load: Can not open file <"<<filename<<">\n";
    return (false);
  }

  if ((suffix == "jpg") || (suffix == "JPG") || (suffix == "jpeg") || (suffix == "JPEG")) {
    im = gdImageCreateFromJpeg(in);
  }

  if ((suffix == "png") || (suffix == "PNG")) {
    im = gdImageCreateFromPng(in);
  }

//#ifdef HAVE_GD2
 if ((suffix == "gif") || (suffix == "GIF")) {
    im = gdImageCreateFromGif(in);
  }
//#endif

  if (im != 0) {

//#ifdef HAVE_GD2
    if ((_width != 0) && (_height != 0)) {

      uint32 origWidth(gdImageSX(im));
      uint32 origHeight(gdImageSY(im));
      /* calculate the new size -> picture must fit into the given rectangle */
      float factorX  = (_width*1.0)/(origWidth*1.0);
      float factorY  = (_height*1.0)/(origHeight*1.0);
      float factor(1.0);

#ifdef DEBUG
      std::cout << "wanted: "<<_width<<"x"<<_height<<"  orig: "<<origWidth<<"x"<<origHeight<<std::endl;
#endif

      if (useBiggest) {
      if (factorX < factorY)
    	  factor = factorY;
      else
        factor = factorX;
      }
      else {
          if (factorX < factorY)
        	  factor = factorX;
          else
            factor = factorY;
      }
#ifdef DEBUG
      std::cerr << "recalculating ("<<factor<<") image to "<<(uint32) (origWidth*factor+0.5)<< "x"<< (uint32) (origHeight*factor+0.5)<<std::endl;
#endif

      gdImagePtr resampled = gdImageCreateTrueColor((uint32) (origWidth*factor+0.5), (uint32) (origHeight*factor+0.5));

      gdImageCopyResampled(resampled, im, 0, 0, 0, 0, resampled->sx, resampled->sy, origWidth, origHeight);

      convertToInternal(resampled);

      gdImageDestroy(resampled);
    }
    else {
      convertToInternal(im);
    }
//#else
//    convertToInternal(im);
//#endif

    gdImageDestroy(im);

    fclose(in);

    return(true);

  }

  fclose(in);

  std::cerr << "RGBPlane::load: Can not read file of type <"<<suffix<<">\n";

  return(false);
}


void RGBPlane::convertToInternal(gdImagePtr im)
{
  width = gdImageSX(im);
  height = gdImageSY(im);
  stride = width;

  cleanup();

  plane = new uint8[width*height*4];

  uint8 alpha=255;

  int c(0);
  uint32 x(0);

  for(uint32 i(0); i<height; ++i)
    for(uint32 j(0); j<width; ++j) {
      c = gdImageGetPixel(im, j, i);
      plane[x++] = gdImageRed(im, c); //im->red[c];
      plane[x++] = gdImageGreen(im, c); //im->green[c];
      plane[x++] = gdImageBlue(im,c); //im->blue[c];
      plane[x++] = alpha;
    }

}

RGBPlane::SuffixType RGBPlane::identifySuffix(const std::string& filename)
{
  std::string::size_type suffixStart(filename.find_last_of('.'));

  if (suffixStart == std::string::npos) {
    std::cerr << "RGBPlane::load: Can not identify suffix of <"<<filename<<">\n";
    return(suffix_unknown);
  }

  std::string suffix(filename.substr(suffixStart+1));

  if ((suffix == "jpg") || (suffix == "JPG") || (suffix == "jpeg") || (suffix == "JPEG")) {
    return(suffix_jpg);
  }

  if ((suffix == "png") || (suffix == "PNG")) {
    return(suffix_png);
  }

//#ifdef HAVE_GD2
 if ((suffix == "gif") || (suffix == "GIF")) {
   return(suffix_gif);
 }
//#endif

 return(suffix_unknown);

}

bool RGBPlane::save(const std::string& filename, uint32 newWidth, uint32 newHeight)
{

  gdImagePtr im;

  SuffixType type = identifySuffix(filename);

  im = gdImageCreateTrueColor(width, height);

  int actColor;
  int planeCount(0);

//  uint32 size(width*height*4);

  for(uint32 i(0); i < height; ++i)
    for(uint32 j(0); j < width; ++j) {
      int red = plane[planeCount++];
      int green = plane[planeCount++];
      int blue  = plane[planeCount++];
      actColor = gdImageColorAllocate(im, red , green , blue);

      planeCount++; // alpha channel not in use

      /* Set a pixel near the center. */
      gdImageSetPixel(im, j, i, actColor);

    }

  FILE* out = fopen(filename.c_str(), "wb");

  if ((newWidth != 0) || (newHeight != 0)) {

    if (newWidth == 0)
      newWidth = width*newHeight/height;

    if (newHeight == 0)
      newHeight = height*newWidth/width;

//    std::cout << "resizing to <"<<newWidth<<":"<<newHeight<<">\n";

    gdImagePtr resampled;
    resampled = gdImageCreateTrueColor(newWidth, newHeight);

    gdImageCopyResampled(resampled, im, 0, 0, 0, 0, resampled->sx, resampled->sy, width, height);

    switch(type) {

    case suffix_jpg:
      gdImageJpeg(resampled, out, -1);
      break;

    case suffix_png:
      gdImagePng(resampled, out);
      break;

//    case suffix_gif:

    default:
      std::cerr << "can not identify suffix\n";

    }
    /* Write JPEG using default quality */


  }
  else {

  /* Write JPEG using default quality */
  gdImageJpeg(im, out, -1);

  }

  /* Close file */
  fclose(out);

  /* Destroy it */
  gdImageDestroy(im);

  return(true);
}


void RGBPlane::cleanup()
{
  delete[] plane;
  plane = 0;
}

uint8 RGBPlane::linearInterpolation(float x, float y, uint32 add)
{
  uint32 pixelDistance = 4;
  uint32 pos_x1 = (int)(x);
  uint32 pos_x2 = (int)(x+1);
  uint32 pos_y1 = (int)(y);
  uint32 pos_y2 = (int)(y+1);

  if (pos_x2 >= width)
    pos_x2 = width-1;

  if (pos_y2 >= height)
    pos_y2 = height-1;

  float part_x = (float)(x - pos_x1);
  float part_y = (float)(y - pos_y1);

  float value_x1y1 = plane[pixelDistance*(pos_y1*width+pos_x1)+add];
  float value_x1y2 = plane[pixelDistance*(pos_y2*width+pos_x1)+add];
  float value_x2y1 = plane[pixelDistance*(pos_y1*width+pos_x2)+add];
  float value_x2y2 = plane[pixelDistance*(pos_y2*width+pos_x2)+add];

  float inter_x1y1_x1y2 = (value_x1y2-value_x1y1)*part_y + value_x1y1;
  float inter_x2y1_x2y2 = (value_x2y2-value_x2y1)*part_y + value_x2y1;

  float endpoint = (inter_x2y1_x2y2 - inter_x1y1_x1y2) * part_x
      + inter_x1y1_x1y2 + 0.5;

  if (endpoint > 255)
    endpoint = 255;

  if (endpoint < 0)
    endpoint = 0;

  return ((uint8) endpoint);
}

RGBPlane RGBPlane::crossfade(RGBPlane& other, float velocity)
{
  if ((other.width != width) || (other.height != height))
    throw("can not crossfade, planes not matching");

  uint32 size = width*height*4;

  RGBPlane retPlane(width, height);

  for(uint32 i(0);i<size;++i) {

    uint32 pixel1 = other.plane[i];
    uint32 pixel2 = plane[i];
    uint32 newPixel = (uint32)(pixel1*(1.0-velocity) + pixel2*velocity);

    if (newPixel > 0xFF)
      newPixel = 0xFF;

    retPlane.plane[i] = (uint8)newPixel;
  }

  return(retPlane);
}

RGBPlane RGBPlane::resize(float scaleFactor)
{
  RGBPlane scaled((uint32)(width * scaleFactor + 0.5), (uint32)(height * scaleFactor+0.5));

  uint32 size(scaled.width*scaled.height*4);

  memset(scaled.plane, 0x00, size);

  float resizeFactor = 1/scaleFactor;

 // std::cout << "size of scaled frame: "<<scaled.width<< "x"<<scaled.height<<std::endl;

  for (uint32 j(0); j< scaled.height; ++j)
    for (uint32 i(0); i < scaled.width; ++i)
      for (uint32 k(0); k<4; ++k)
        scaled.plane[4*(j*scaled.width+i)+k] = linearInterpolation((i*1.0)*resizeFactor, (j
            *1.0)*resizeFactor, k);

  return(scaled);
}

RGBPlane RGBPlane::reframe(uint32 newWidth, uint32 newHeight)
{
  RGBPlane scaledPlane; // only temporary
  RGBPlane newPlane;

  newPlane.width = newWidth;
  newPlane.stride = newWidth;
  newPlane.height = newHeight;
  newPlane.format = rgba;

  // place the new picture into the original frame
  uint32 planesize(newWidth*newHeight*4);
  newPlane.plane = new uint8[planesize];

  // fill the plane with the given (actually black) background
  memset(newPlane.plane, 0x00, planesize);
  // setBackground

  uint32 offsetY(0);
  uint32 offsetX(0);
  float scaleFactor(1);

  if ((height*newPlane.width*1.0)/(width*1.0)
      < (newPlane.height*1.0)) {
    // we work with a height offset
    offsetY = (uint32) (((newPlane.height*1.0) - ((height
        *newPlane.width*1.0)/(width*1.0)))/2.0);
    offsetX = 0;
    scaleFactor = (newPlane.width*1.0)/(width*1.0);
  } else {
    // we work with a width offset
    offsetY = 0;
    offsetX = (uint32) (((newPlane.width*1.0) - ((width
        *newPlane.height*1.0)/(height*1.0)))/2.0);
    scaleFactor = (newPlane.height*1.0)/(height*1.0);
  }
//  std::cout << "original size: "<< width<<"x"<<height<<std::endl;
//  std::cout << "rescale factor: "<<scaleFactor<<"   offsetX: "<<offsetX<<"   offsetY: "<<offsetY<<std::endl;

  scaledPlane = resize(scaleFactor);

  uint32 position_new;
  uint32 position_scaled;

  // place the picture into the new frame
  for (uint32 i(0); i<scaledPlane.width; ++i) {
    for (uint32 j(0); j <scaledPlane.height; ++j) {
      for (int32 k(0); k < 4; ++k) {
      position_new = 4*((j+offsetY)*newPlane.width+offsetX+i)+k;
      position_scaled = 4*(j*scaledPlane.width+i)+k;
      newPlane.plane[position_new] = scaledPlane.plane[position_scaled];
      }
    }
  }

  scaledPlane.cleanup();

  return(newPlane);
}

RGBPlane RGBPlane::subframe(uint32 newWidth, uint32 newHeight, float offsetWidth, float offsetHeight, float scaleFactor)
{
  RGBPlane resized;

  resized.width = newWidth;
  resized.height = newHeight;
  resized.stride = newWidth;

  if ((((newWidth*1.0)/scaleFactor)+offsetWidth > width) || (((newHeight*1.0)/scaleFactor)+offsetHeight > height)) {
    std::cerr << "new width: "<<newWidth<<" / "<<scaleFactor<<" + "<<offsetWidth << " = " << ((newWidth*1.0)/scaleFactor)+offsetWidth
        <<" must be smaller then "<<width<<std::endl;
    std::cerr << "new width: "<<newHeight<<" / "<< scaleFactor<<" + "<<offsetHeight << " = " <<((newHeight*1.0)/scaleFactor)+offsetHeight
        <<" must be smaller then "<<height<<std::endl;
    throw "PicConverter::subPic: new width/height is/are to big";
  }

  uint32 size(newWidth*newHeight*4);
  resized.plane = new uint8[size];

  for (uint32 j(0); j < newHeight; ++j)
    for (uint32 i(0); i < newWidth; ++i)
      for (uint32 k(0); k<4; ++k)
        resized.plane[4*(j*newWidth+i)+k] = linearInterpolation((i*1.0)/scaleFactor+offsetWidth, (j*1.0)/scaleFactor
            +offsetHeight, k);

  return(resized);

}

#ifdef HAVE_LIBTHEORAENC
//#ifdef HAVE_LIBTHEORA
void RGBPlane::exportYCrCb_theora(th_ycbcr_buffer& buffer) const
{
  if ((width  != (uint32)buffer[0].width) ||
      (height != (uint32)buffer[0].height)) {

    /* delete old planes */
    delete buffer[0].data;
    delete buffer[1].data;
    delete buffer[2].data;

    /* create a new YCbCrPlane */
    buffer[0].width = width;
    buffer[0].height = height;
    buffer[0].stride = stride;
    buffer[0].data = new uint8[width*height];

    buffer[1].width = width/2;
    buffer[1].height = height/2;
    buffer[1].stride = stride/2;
    buffer[1].data = new uint8[width*height/4];

    buffer[2].width = width/2;
    buffer[2].height = height/2;
    buffer[2].stride = stride/2;
    buffer[2].data = new uint8[width*height/4];

  }

  /* convert to the "old" algorithm */
  uint8 *lum = buffer[0].data;
  uint8 *cb = buffer[1].data;
  uint8 *cr = buffer[2].data;

  uint8* src = plane;

  int wrap, wrap3, x, y;
  int r, g, b, r1, g1, b1;
  uint8* p;

  wrap = width;
  wrap3 = stride * 4;
  p = src;
  for (y=0; y<(int)height; y+=2) {
    for (x=0; x<(int)width; x+=2) {
      r = p[0];
      g = p[1];
      b = p[2];
      r1 = r;
      g1 = g;
      b1 = b;
      lum[0] = (FIX(0.29900) * r + FIX(0.58700) * g +FIX(0.11400) * b
      + ONE_HALF) >> SCALEBITS;

      r = p[4];
      g = p[5];
      b = p[6];
      r1 += r;
      g1 += g;
      b1 += b;
      lum[1] = (FIX(0.29900) * r + FIX(0.58700) * g +FIX(0.11400) * b
      + ONE_HALF) >> SCALEBITS;

      // jump to the next line (we have to collect all 4 values for the cb/cr value)
      p += wrap3;
      lum += wrap;

      r = p[0];
      g = p[1];
      b = p[2];
      r1 += r;
      g1 += g;
      b1 += b;
      lum[0] = (FIX(0.29900) * r + FIX(0.58700) * g +FIX(0.11400) * b
      + ONE_HALF) >> SCALEBITS;

      r = p[4];
      g = p[5];
      b = p[6];
      r1 += r;
      g1 += g;
      b1 += b;
      lum[1] = (FIX(0.29900) * r + FIX(0.58700) * g +FIX(0.11400) * b
      + ONE_HALF) >> SCALEBITS;

      cb[0] = ((-FIX(0.16874) * r1 - FIX(0.33126) * g1+FIX(0.50000) * b1+ 4* ONE_HALF- 1) >> (SCALEBITS + 2)) + 128;
      cr[0] = ((FIX(0.50000) * r1 - FIX(0.41869) * g1 -FIX(0.08131) * b1 + 4
      * ONE_HALF - 1) >> (SCALEBITS + 2)) + 128;
      cb++;
      cr++;

      // jump back one line and then forth two pixel
      p += -wrap3 + 2 * 4;
      lum += -wrap + 2;
    }

    p += wrap3;
    lum += wrap;
  }

}

void RGBPlane::importYCrCb_theora(const th_ycbcr_buffer& buffer)
{
  // we assume a YCbCr 420

#define min(a,b)((a<b)?a:b)
#define max(a,b)((a<b)?b:a)

  width = buffer[0].width;
  height = buffer[0].height;
  stride = buffer[0].stride;

  uint32 size(width*height*4);

  uint8* YPlane  = buffer[0].data;
  uint8* CrPlane = buffer[2].data;
  uint8* CbPlane = buffer[1].data;

  delete[] plane;
  plane = new uint8[size];

  // using cr plain as reference
  for(uint32 i(0); i<(uint32)buffer[1].height; ++i)
    for(uint32 j(0); j<(uint32)buffer[1].width; ++j) {

      uint32 position00 = (2*i)*buffer[0].stride+(2*j);
      uint32 position01 = (2*i)*buffer[0].stride+(2*j+1);
      uint32 position10 = (2*i+1)*buffer[0].stride+(2*j);
      uint32 position11 = (2*i+1)*buffer[0].stride+(2*j+1);

      uint32 outPos00 = 4*((2*i)*width+(2*j));
      uint32 outPos01 = 4*((2*i)*width+(2*j+1));
      uint32 outPos10 = 4*((2*i+1)*width+(2*j));
      uint32 outPos11 = 4*((2*i+1)*width+(2*j+1));

      uint32 cromaPos = i*buffer[1].stride+j;

      /* red */
      int32 value;

      value = (uint32)(YPlane[position00] + (1.402 * (CrPlane[cromaPos] - 128)));
      value = min(255,value);
      value = max(0,value);
      plane[outPos00] = value;

      value = (uint32)(YPlane[position01] + (1.402 * (CrPlane[cromaPos] - 128)));
      value = min(255,value);
      value = max(0,value);
      plane[outPos01] = value;

      value = (uint32)(YPlane[position10] + (1.402 * (CrPlane[cromaPos] - 128)));
      value = min(255,value);
      value = max(0,value);
      plane[outPos10] = value;

      value = (uint32)(YPlane[position11] + (1.402 * (CrPlane[cromaPos] - 128)));
      value = min(255,value);
      value = max(0,value);
      plane[outPos11] = value;

//      plane[outPos00] = YPlane[position00] + (1.402 * (CrPlane[cromaPos] - 128));
//      plane[outPos01] = YPlane[position01] + (1.402 * (CrPlane[cromaPos] - 128));
//      plane[outPos10] = YPlane[position10] + (1.402 * (CrPlane[cromaPos] - 128));
//      plane[outPos11] = YPlane[position11] + (1.402 * (CrPlane[cromaPos] - 128));

      /* green */
      value = (uint32)(YPlane[position00] - (0.34414 * (CbPlane[cromaPos] - 128)) - (0.71414 * (CrPlane[cromaPos] - 128)));
      value = min(255,value);
      value = max(0,value);
      plane[outPos00+1] = value;

      value = (uint32)(YPlane[position01] - (0.34414 * (CbPlane[cromaPos] - 128)) - (0.71414 * (CrPlane[cromaPos] - 128)));
      value = min(255,value);
      value = max(0,value);
      plane[outPos01+1] = value;

      value = (uint32)(YPlane[position10] - (0.34414 * (CbPlane[cromaPos] - 128)) - (0.71414 * (CrPlane[cromaPos] - 128)));
      value = min(255,value);
      value = max(0,value);
      plane[outPos10+1] = value;

      value = (uint32)(YPlane[position11] - (0.34414 * (CbPlane[cromaPos] - 128)) - (0.71414 * (CrPlane[cromaPos] - 128)));
      value = min(255,value);
      value = max(0,value);
      plane[outPos11+1] = value;

//      plane[outPos00+1] = YPlane[position00] - (0.34414 * (CbPlane[cromaPos] - 128)) - (0.71414 * (CrPlane[cromaPos] - 128));
//      plane[outPos01+1] = YPlane[position01] - (0.34414 * (CbPlane[cromaPos] - 128)) - (0.71414 * (CrPlane[cromaPos] - 128));
//      plane[outPos10+1] = YPlane[position10] - (0.34414 * (CbPlane[cromaPos] - 128)) - (0.71414 * (CrPlane[cromaPos] - 128));
//      plane[outPos11+1] = YPlane[position11] - (0.34414 * (CbPlane[cromaPos] - 128)) - (0.71414 * (CrPlane[cromaPos] - 128));

      /* blue */
      value = (uint32)(YPlane[position00] + (1.772 * (CbPlane[cromaPos] - 128)));
      value = min(255,value);
      value = max(0,value);
      plane[outPos00+2] = value;

      value = (uint32)(YPlane[position00] + (1.772 * (CbPlane[cromaPos] - 128)));
      value = min(255,value);
      value = max(0,value);
      plane[outPos01+2] = value;

      value = (uint32)(YPlane[position00] + (1.772 * (CbPlane[cromaPos] - 128)));
      value = min(255,value);
      value = max(0,value);
      plane[outPos10+2] = value;

      value = (uint32)(YPlane[position00] + (1.772 * (CbPlane[cromaPos] - 128)));
      value = min(255,value);
      value = max(0,value);
      plane[outPos11+2] = value;


//      plane[outPos00+2] = YPlane[position00] + (1.772 * (CbPlane[cromaPos] - 128));
//      plane[outPos01+2] = YPlane[position01] + (1.772 * (CbPlane[cromaPos] - 128));
//      plane[outPos10+2] = YPlane[position10] + (1.772 * (CbPlane[cromaPos] - 128));
//      plane[outPos11+2] = YPlane[position11] + (1.772 * (CbPlane[cromaPos] - 128));

      plane[outPos00+3] = 255;
      plane[outPos01+3] = 255;
      plane[outPos10+3] = 255;
      plane[outPos11+3] = 255;

    }

}

#endif

RGBPlane& RGBPlane::operator =(const RGBPlane& rgbPlane)
{
	if (this != &rgbPlane) {
		cleanup();
		width  = rgbPlane.width;
		height = rgbPlane.height;
		stride = rgbPlane.stride;
		format = rgbPlane.format;
		plane  = rgbPlane.plane;
	}
	return(*this);
}

uint8 YCbCrPlane::linearInterpolation(float x, float y, PicPlane& localPlane)
{
  uint32 pos_x1 = (int)(x);
  uint32 pos_x2 = (int)(x+1);
  uint32 pos_y1 = (int)(y);
  uint32 pos_y2 = (int)(y+1);

  if (pos_x2 >= localPlane.stride)
    pos_x2 = localPlane.stride-1;

  if (pos_y2 >= localPlane.height)
    pos_y2 = localPlane.height-1;

  float part_x = (float)(x - pos_x1);
  float part_y = (float)(y - pos_y1);

  float value_x1y1 = localPlane.plane[pos_y1*localPlane.stride+pos_x1];
  float value_x1y2 = localPlane.plane[pos_y2*localPlane.stride+pos_x1];
  float value_x2y1 = localPlane.plane[pos_y1*localPlane.stride+pos_x2];
  float value_x2y2 = localPlane.plane[pos_y2*localPlane.stride+pos_x2];

  float inter_x1y1_x1y2 = (value_x1y2-value_x1y1)*part_y + value_x1y1;
  float inter_x2y1_x2y2 = (value_x2y2-value_x2y1)*part_y + value_x2y1;

  float endpoint = (inter_x2y1_x2y2 - inter_x1y1_x1y2) * part_x
      + inter_x1y1_x1y2 + 0.5;

  if (endpoint > 255)
    endpoint = 255;

  if (endpoint < 0)
    endpoint = 0;

  return ((uint8) endpoint);
}


YCbCrPlane YCbCrPlane::resize(float scaleFactor)
{
	YCbCrPlane scaled((uint32)(width * scaleFactor + 0.5), (uint32)(height * scaleFactor+0.5));

	uint32 fullPlaneSize(scaled.width*scaled.height);
	memset(scaled.yPlane.plane, 0x00, fullPlaneSize);
	memset(scaled.crPlane.plane, 0x00, fullPlaneSize/4);
	memset(scaled.cbPlane.plane, 0x00, fullPlaneSize/4);
	float resizeFactor = 1/scaleFactor;

//	std::cerr << "size of scaled frame: "<<scaled.width<< "x"<<scaled.height<<std::endl;

	// recalculate the Y-plane
	for (uint32 j(0); j< scaled.height; ++j)
	  for (uint32 i(0); i < scaled.width; ++i)
	    scaled.yPlane.plane[(j*scaled.stride+i)] = linearInterpolation((i*1.0)*resizeFactor, (j*1.0)*resizeFactor, yPlane);

	// recalculate the Cb-plane
	for (uint32 j(0); j< scaled.height/2; ++j)
	  for (uint32 i(0); i < scaled.width/2; ++i) {
	    scaled.cbPlane.plane[4*(j*scaled.stride/2+i)] = linearInterpolation((i*1.0)*resizeFactor, (j*1.0)*resizeFactor, yPlane);
	    scaled.crPlane.plane[4*(j*scaled.stride/2+i)] = linearInterpolation((i*1.0)*resizeFactor, (j*1.0)*resizeFactor, yPlane);
	  }

	return(scaled);

}

void YCbCrPlane::importYCbCr_theora(const th_ycbcr_buffer& buffer)
{
	  yPlane.width  = buffer[0].width;
	  yPlane.height = buffer[0].height;
	  yPlane.stride = buffer[0].stride;
	  yPlane.plane  = buffer[0].data;

	  cbPlane.width  = buffer[1].width;
	  cbPlane.height = buffer[1].height;
	  cbPlane.stride = buffer[1].stride;
	  cbPlane.plane  = buffer[1].data;

	  crPlane.width  = buffer[2].width;
	  crPlane.height = buffer[2].height;
	  crPlane.stride = buffer[2].stride;
	  crPlane.plane  = buffer[2].data;

}

void YCbCrPlane::exportYCbCr_theora(th_ycbcr_buffer& buffer) const
{
	  buffer[0].width  = yPlane.width;
	  buffer[0].height = yPlane.height;
	  buffer[0].stride = yPlane.stride;
	  buffer[0].data   = yPlane.plane;

	  buffer[1].width  = cbPlane.width;
	  buffer[1].height = cbPlane.height;
	  buffer[1].stride = cbPlane.stride;
	  buffer[1].data   = cbPlane.plane;

	  buffer[2].width  = crPlane.width;
	  buffer[2].height = crPlane.height;
	  buffer[2].stride = crPlane.stride;
	  buffer[2].data   = crPlane.plane;
}

void YCbCrPlane::cleanup()
{
	delete yPlane.plane;
	delete cbPlane.plane;
	delete crPlane.plane;

	yPlane.plane  = 0;
	cbPlane.plane = 0;
	crPlane.plane = 0;

}